package javax.management.relation;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.io.ObjectOutputStream;
import java.io.IOException;
import com.sun.jmx.mbeanserver.Util;
import java.io.ObjectInputStream;
import java.util.logging.Level;
import javax.management.MBeanServerNotification;
import javax.management.Notification;
import com.sun.jmx.defaults.JmxProperties;
import java.util.Vector;
import javax.management.ObjectName;
import java.util.List;
import java.io.ObjectStreamField;
import javax.management.NotificationFilterSupport;

public class MBeanServerNotificationFilter extends NotificationFilterSupport
{
    private static final long oldSerialVersionUID = 6001782699077323605L;
    private static final long newSerialVersionUID = 2605900539589789736L;
    private static final ObjectStreamField[] oldSerialPersistentFields;
    private static final ObjectStreamField[] newSerialPersistentFields;
    private static final long serialVersionUID;
    private static final ObjectStreamField[] serialPersistentFields;
    private static boolean compat;
    private List<ObjectName> selectedNames;
    private List<ObjectName> deselectedNames;
    
    public MBeanServerNotificationFilter() {
        this.selectedNames = new Vector();
        this.deselectedNames = null;
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "MBeanServerNotificationFilter");
        this.enableType("JMX.mbean.registered");
        this.enableType("JMX.mbean.unregistered");
        JmxProperties.RELATION_LOGGER.exiting(MBeanServerNotificationFilter.class.getName(), "MBeanServerNotificationFilter");
    }
    
    public synchronized void disableAllObjectNames() {
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "disableAllObjectNames");
        this.selectedNames = new Vector();
        this.deselectedNames = null;
        JmxProperties.RELATION_LOGGER.exiting(MBeanServerNotificationFilter.class.getName(), "disableAllObjectNames");
    }
    
    public synchronized void disableObjectName(ObjectName param1) throws IllegalArgumentException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "disableObjectName", param1);
        if (this.selectedNames != null && this.selectedNames.size() != 0) {
            this.selectedNames.remove(param1);
        }
        if (this.deselectedNames != null && !this.deselectedNames.contains(param1)) {
            this.deselectedNames.add(param1);
        }
        JmxProperties.RELATION_LOGGER.exiting(MBeanServerNotificationFilter.class.getName(), "disableObjectName");
        return;
    }
    
    public synchronized void enableAllObjectNames() {
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "enableAllObjectNames");
        this.selectedNames = null;
        this.deselectedNames = new Vector();
        JmxProperties.RELATION_LOGGER.exiting(MBeanServerNotificationFilter.class.getName(), "enableAllObjectNames");
    }
    
    public synchronized void enableObjectName(ObjectName param1) throws IllegalArgumentException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "enableObjectName", param1);
        if (this.deselectedNames != null && this.deselectedNames.size() != 0) {
            this.deselectedNames.remove(param1);
        }
        if (this.selectedNames != null && !this.selectedNames.contains(param1)) {
            this.selectedNames.add(param1);
        }
        JmxProperties.RELATION_LOGGER.exiting(MBeanServerNotificationFilter.class.getName(), "enableObjectName");
        return;
    }
    
    public synchronized Vector<ObjectName> getEnabledObjectNames() {
        if (this.selectedNames != null) {
            return new Vector(this.selectedNames);
        }
        return null;
    }
    
    public synchronized Vector<ObjectName> getDisabledObjectNames() {
        if (this.deselectedNames != null) {
            return new Vector(this.deselectedNames);
        }
        return null;
    }
    
    @Override
    public synchronized boolean isNotificationEnabled(Notification param1) throws IllegalArgumentException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", param1);
        if (!this.getEnabledTypes().contains(param1.getType())) {
            JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "Type not selected, exiting");
            return false;
        }
        ObjectName mBeanName = ((MBeanServerNotification)param1).getMBeanName();
        boolean contains = false;
        if (this.selectedNames != null) {
            if (this.selectedNames.size() == 0) {
                JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "No ObjectNames selected, exiting");
                return false;
            }
            contains = this.selectedNames.contains(mBeanName);
            if (contains == 0) {
                JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "ObjectName not in selected list, exiting");
                return false;
            }
        }
        if (contains == 0) {
            if (this.deselectedNames == null) {
                JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "ObjectName not selected, and all names deselected, exiting");
                return false;
            }
            if (this.deselectedNames.contains(mBeanName)) {
                JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "ObjectName explicitly not selected, exiting");
                return false;
            }
        }
        JmxProperties.RELATION_LOGGER.logp(Level.FINER, MBeanServerNotificationFilter.class.getName(), "isNotificationEnabled", "ObjectName selected, exiting");
        return true;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        if (MBeanServerNotificationFilter.compat) {
            ObjectInputStream.GetField fields = objectInputStream.readFields();
            this.selectedNames = (List)Util.cast(fields.get("mySelectObjNameList", null));
            if (fields.defaulted("mySelectObjNameList")) {
                throw new NullPointerException("mySelectObjNameList");
            }
            this.deselectedNames = (List)Util.cast(fields.get("myDeselectObjNameList", null));
            if (fields.defaulted("myDeselectObjNameList")) {
                throw new NullPointerException("myDeselectObjNameList");
            }
        }
        else {
            objectInputStream.defaultReadObject();
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (MBeanServerNotificationFilter.compat) {
            ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
            putFields.put("mySelectObjNameList", this.selectedNames);
            putFields.put("myDeselectObjNameList", this.deselectedNames);
            objectOutputStream.writeFields();
        }
        else {
            objectOutputStream.defaultWriteObject();
        }
    }
    
    static {
        oldSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("mySelectObjNameList", Vector.class), new ObjectStreamField("myDeselectObjNameList", Vector.class) };
        newSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("selectedNames", List.class), new ObjectStreamField("deselectedNames", List.class) };
        MBeanServerNotificationFilter.compat = false;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.serial.form"));
            MBeanServerNotificationFilter.compat = (s != null && s.equals("1.0"));
        }
        catch (Exception ex) {}
        if (MBeanServerNotificationFilter.compat) {
            serialPersistentFields = MBeanServerNotificationFilter.oldSerialPersistentFields;
            serialVersionUID = 6001782699077323605L;
        }
        else {
            serialPersistentFields = MBeanServerNotificationFilter.newSerialPersistentFields;
            serialVersionUID = 2605900539589789736L;
        }
    }
}
